#include "includes/common_functions.inc"

// * User input * //
uniform mediump sampler2D u_texture;
uniform mediump sampler2D u_texture1;
uniform mediump sampler2D u_texture2;
uniform mediump sampler2D u_texture3;
	
// x = hue shift, range 0-1
// y = saturation shift, range -1.0 - 1.0 
// z = brightness shift, range -1.0 - 1.0 
uniform mediump vec4 u_HSV_shift; 


//Fog colors
//Near fog color
uniform mediump vec4 u_materialAmbient;
//Far fog color
uniform mediump vec4 u_materialDiffuse;

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor; 

// Data IN FROM VERTEX SHADER
varying lowp vec3 v_ambient_colour;
varying highp vec3 v_normal;
varying highp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec2 v_texCoord2;
varying lowp vec3 v_camDir;
varying mediump float v_fog_intensity;
varying mediump float v_dist;
varying mediump float v_length;
varying mediump vec3 v_cam_pos;


void main() {
	// NORMALIZE INPUT VALUES TO ENSURE COUNTER INTERPOLATION ISSUES
	highp vec3 L = normalize(-u_directionalLightMatrix[2].xyz);
	highp vec3 V = normalize(v_camDir);
	highp vec3 N = normalize(v_normal);
	
	// ALBEDO - DIFFUSE TEXTURE READ
	mediump vec4 albedo = texture2D(u_texture, v_texCoord0);
	albedo.rgb = pow(albedo.rgb, vec3(2.2, 2.2, 2.2));
	
	// DETAIL MAP - TEXTURE READ
	mediump vec4 detail = texture2D(u_texture1, v_texCoord1);
	//mediump vec4 detail = vec4(0.6,0.0,0.0,1.0);
	detail.rgb = pow(detail.rgb, vec3(2.2, 2.2, 2.2));

	//HSV SHIFTED DETAILS
	albedo.rgb = hsvShiftDetail(albedo.rgb, detail.r, u_HSV_shift.xyz);
	
	// DOT PRODUCTS + REFLECTION VECTORS
	highp float NdotV = dot(N, V);
	highp float NdotL = max(0.0, dot(N, L));
	highp vec3 R = reflect(-V, N);
	highp float H = clamp(dot(L, R), 0.0, 1.0);
	
	// LIGHTING CALCLUATIONS
	lowp vec3 dir_light = u_directionalLightColor.rgb * NdotL * u_directionalLightColor.a * albedo.a;
	
	// LIGHTMAP - LIGHTMAP TEXTURE READ
	mediump vec4 lightMap = texture2D(u_texture2, v_texCoord2);
	lightMap.rgb = pow(lightMap.rgb, vec3(2.2, 2.2, 2.2));
	
	albedo.rgb = albedo.rgb * clamp(lightMap.rgb, v_ambient_colour, vec3(1.0,1.0,1.0));	
	//albedo.rgb = albedo.rgb * lightMap.rgb;	
	
	// PHYSICALLY CORRECT SHADING 
	highp vec3 mrw = vert_CalculateMRW(v_vert_color.rgb);
	
	// SPEC APPROXIMATION  
	highp vec3 specColour = mix(vec3(0.03, 0.03, 0.03), albedo.rgb * 2.0, mrw.r);
	
	highp vec2 sphericalCoords = vert_CalculateSphericalCoords(mrw.g, R);
	
	//ENV SAMPLING
	highp vec4 envMapTex = texture2D(u_texture3, sphericalCoords);
	envMapTex = pow(envMapTex, vec4(2.2, 2.2, 2.2, 2.2));
	
	//Function CalculateEnvSpec accepts parameters (vec4 envMap, vec3 mrw, float NdotV, vec3 amb_color, vec3 spec_color)
	//Returns vec4 envSpec, env map colors in rgb, spec coefficient in a.
	highp vec4 envSpec = CalculateEnvSpec(envMapTex, mrw, NdotV, v_ambient_colour, specColour);
	
	highp vec3 envMap = envSpec.rgb;
	highp vec3 spec_BRDF = specColour * dir_light * envSpec.a;
	
	//FOG
	// Blend between two fog colors based on distance
	mediump vec3 fog_color = mix(u_materialAmbient.rgb * u_materialAmbient.a, u_materialDiffuse.rgb * u_materialDiffuse.a, v_fog_intensity);
	
	// BUILD FRAGMENT OUTPUT  
	highp vec4 returnValue = vec4(albedo.rgb, 1.0);
	returnValue.rgb *= dir_light + (v_ambient_colour * albedo.a);
	returnValue.rgb += spec_BRDF.rgb * albedo.a;
	returnValue.rgb += envMap.rgb * albedo.a;
	returnValue.rgb = returnValue.rgb * (1.0 - v_fog_intensity) + (fog_color.rgb * v_fog_intensity);
	
	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = pow(returnValue.rgb, vec3(1.0/2.2, 1.0/2.2, 1.0/2.2));
	gl_FragColor = returnValue;
	
	//Debug
	//gl_FragColor = vec4(v_vert_color.rgb, 1.0);
	//gl_FragColor = vec4(v_normal, 1.0);
	//gl_FragColor = vec4(envMap.rgb, 1.0);
	//gl_FragColor = vec4(sphericalCoords, 0.0, 1.0);
	//gl_FragColor = vec4(v_texCoord0, 0.0, 1.0);
	//gl_FragColor = vec4(v_texCoord1, 0.0, 1.0);
	//gl_FragColor = vec4(v_texCoord2, 0.0, 1.0);
	//gl_FragColor.rgb = lightMap.rgb;
	//gl_FragColor.rgb = vec3(v_texCoord1,1.0);
	//gl_FragColor.rgb = spec_BRDF.rgb;
}