// * User input * //
uniform mediump sampler2D u_texture;
uniform mediump sampler2D u_texture1;
uniform mediump sampler2D u_texture2;

// x = hue shift, range 0-1
// y = saturation shift, range -1.0 - 1.0 
// z = brightness shift, range -1.0 - 1.0 
uniform mediump vec4 u_HSV_shift; 

// Data IN FROM VERTEX SHADER
varying lowp vec3 v_ambient_colour;
varying mediump vec3 v_normal;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;

varying lowp vec3 v_dir_light;
varying lowp vec3 v_metallic_roughness;
varying lowp float v_spec_coefficient;
varying lowp float v_env_map_coefficient;
varying lowp vec2 v_AB;
varying lowp vec2 v_spherical_coords;

vec3 rgb2hsv(vec3 c)
{
    mediump vec4 K = vec4(0.0, -0.3333, 0.6666, -1.0);
    mediump vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    mediump vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}
vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
	
	// NORMALIZE INPUT VALUES TO COUNTER INTERPOLATION ISSUES
	mediump vec3 N = normalize(v_normal);
	
	// ALBEDO - DIFFUSE TEXTURE READ
	mediump vec4 albedo = texture2D(u_texture, v_texCoord0);
	albedo.rgb = pow(albedo.rgb, vec3(2.2, 2.2, 2.2));
	
	// DETAIL MAP - TEXTURE READ
	mediump vec4 detail = texture2D(u_texture1, v_texCoord1);
	detail.rgb = pow(detail.rgb, vec3(2.2, 2.2, 2.2));	
	
	mediump vec3 HSVColour = rgb2hsv(albedo.rgb);

	HSVColour.r -= u_HSV_shift.x;				
	HSVColour.g += u_HSV_shift.y;
	HSVColour.b += u_HSV_shift.z;
	
	HSVColour.g = clamp(HSVColour.g, 0.0, 1.0);
	HSVColour.b = clamp(HSVColour.b, 0.0, 1.0);
	
	mediump vec3 shifted = hsv2rgb(HSVColour);
	
	albedo.rgb = mix(shifted.rgb, albedo.rgb, detail.r);
	
	// LIGHTING CALCLUATIONS
	lowp vec3 directional_light = v_dir_light * albedo.a;
		
	// SPEC APPROXIMATION 
	lowp vec3 specColour = mix(vec3(0.03, 0.03, 0.03), albedo.rgb * 2.0, v_metallic_roughness.r);
	lowp float specular = v_spec_coefficient * albedo.a;
	
	// BRDF
	lowp vec3 spec_BRDF = (directional_light * specular *specColour* v_AB.x + v_AB.y) * (N.y + 1.0) * 0.5;
	
	// ENV MAP
	lowp vec3 envMap = texture2D(u_texture2, v_spherical_coords).rgb;
	envMap.rgb = pow(envMap.rgb, vec3(2.2, 2.2, 2.2));
	envMap.rgb *= v_env_map_coefficient * specColour;
	envMap = clamp(envMap * 6.28 * v_ambient_colour, 0.0, 1.0) * v_metallic_roughness.b;
	
	// BUILD FRAGMENT OUTPUT  -------------------------------------------------------------------------
	lowp vec4 returnValue = vec4(albedo.rgb, 1.0);
	returnValue.rgb *= directional_light + (v_ambient_colour * albedo.a);
	returnValue.rgb += spec_BRDF.rgb * albedo.a;
	returnValue.rgb += envMap.rgb * albedo.a;
	
	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = pow(returnValue.rgb, vec3(1.0/2.2, 1.0/2.2, 1.0/2.2));
	gl_FragColor = returnValue;
}