// * User input * //
uniform mediump sampler2D u_texture; // Diffuse map
uniform mediump sampler2D u_texture1; // Normal map
uniform mediump sampler2D u_texture2; // R = spec map, G = gloss map, B = metal mask
uniform mediump vec4 u_lighting_settings; // x = diffuse wrap, y = shadow strength, z = unused, w = unused
// * --------- * //

uniform highp mat4 u_directionalLightMatrix; // Light matrix in world space
uniform lowp vec4 u_directionalLightColor; // Dir Color in RGB , intensity in A
uniform lowp vec4 u_ambientLightColor;  // Ambient Color in RGB , intensity in A

uniform lowp vec4 u_materialSpecular;  // Specular Color in RGB , intensity in A
uniform lowp vec4 u_materialEmissive;

varying mediump vec2 v_texCoord;
varying mediump vec2 v_texCoord2;
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;
varying mediump vec3 v_camDir;
varying mediump vec4 v_color;

void main() {
    // * VERT INPUTS * //

    mediump vec3 N = normalize(v_normal);
    mediump vec3 T = normalize(v_tangent);
    mediump vec3 B = normalize(v_binormal);
    mediump vec3 V = normalize(v_camDir);
    mediump vec3 L = normalize(-u_directionalLightMatrix[2].xyz);

    // * TEXTURE SAMPLING * //
    mediump vec4 baseColor = texture2D(u_texture, v_texCoord2);
	baseColor = pow(baseColor, vec4(2.2, 2.2, 2.2, 2.2)); // Linear color
    mediump vec3 normalmap = texture2D(u_texture1, v_texCoord).xyz * 2.0 - 1.0;
    mediump vec4 specGloss = texture2D(u_texture2, v_texCoord);
    specGloss = pow(specGloss, vec4(2.2, 2.2, 2.2, 2.2)); // Linear color

    // * NORMAL * //
    mediump mat3 TBN = mat3(T, B, N);
	mediump vec3 Nm = normalize(TBN * normalmap);

    // * DIFFUSE * //
	mediump float diffuseIntensity = clamp(dot(Nm, L), 0.0, 1.0);

	// Diffuse Wrap
	diffuseIntensity = ( diffuseIntensity + u_lighting_settings.x ) / ( 1.0 + u_lighting_settings.x );

	// Shadow Strength
	diffuseIntensity = clamp( diffuseIntensity, u_lighting_settings.y, 1.0 ) * u_directionalLightColor.a;

	mediump vec4 diffuseColor = u_directionalLightColor * diffuseIntensity;
	mediump vec4 ambientColor = u_ambientLightColor * u_ambientLightColor.a;

	//* rimlight *//
	float rm = pow(1.0-dot(N, V),u_materialEmissive.a*25.0);
	vec3 dotNmV2 = vec3(rm, rm, rm);
	vec3 up = vec3(-0.5,0.5,0.0);
	dotNmV2 = max(dotNmV2 * dot(up,Nm),0.0);
	float rimLight = dotNmV2.x;
	rimLight *= v_color.x;

    // * SPECULAR * //
	mediump float specularGlossiness = 255.0 * specGloss.g;
	mediump vec3 H = normalize(V + L);
	mediump float NdotH = max(dot(Nm, H), 0.0);

    vec3 specularColor = mix(vec3(1,1,1), u_materialSpecular.rgb, specGloss.b); // this lets us control what gets colored spec
	mediump vec3 specular = specularColor * specGloss.r * pow(NdotH, specularGlossiness) * u_materialSpecular.a;

	// * FINAL COMP * //
	mediump vec4 ret = vec4(0.0,0.0,0.0,1.0);
	ret = ret + diffuseColor;
	ret = ret + ambientColor;
	ret = ret * baseColor;
	ret.rgb = ret.rgb + specular;
    ret.rgb = ret.rgb + u_materialEmissive.rgb * rimLight;

    ret = pow(ret, vec4(1.0/2.2, 1.0/2.2, 1.0/2.2, 1.0/2.2)); //Back to sRGB

    gl_FragColor.rgb = ret.rgb;
    gl_FragColor.a = baseColor.a;

}

