#include "includes/common_functions.inc"
#include "includes/fog.inc"
#include "includes/close_darken.inc"

// * User input * //
uniform lowp sampler2D u_texture;
uniform lowp sampler2D u_texture1;
uniform lowp sampler2D u_texture2;
uniform lowp sampler2D u_texture3;
	
// x = hue shift, range 0-1
// y = saturation shift, range -1.0 - 1.0 
// z = brightness shift, range -1.0 - 1.0 
uniform mediump vec4 u_HSV_shift; 
uniform mediump vec4 lightmap_color;

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor; 
// Ambient Color in RGB, intensity in A (contrast)
uniform mediump vec4 u_ambientLightColor;

// Data IN FROM VERTEX SHADER
varying lowp vec3 v_ambient_colour;
varying mediump vec3 v_normal;
varying lowp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec2 v_texCoord2;
varying lowp vec3 v_camDir;
varying lowp vec2 v_sphericalCoords;
varying lowp float v_closeDarken;

//varying lowp float v_fogIntensity;

void main() {
	// NORMALIZE INPUT VALUES TO ENSURE COUNTER INTERPOLATION ISSUES
	mediump vec3 L = normalize(-u_directionalLightMatrix[2].xyz);
	mediump vec3 V = normalize(v_camDir);
	mediump vec3 N = normalize(v_normal);

	// DOT PRODUCTS + REFLECTION VECTORS
	float NdotV = dot(N, V);
	float NdotL = max(0.0001, dot(L, N));
	
	// LIGHTING CALCLUATIONS
	lowp vec3 dir_light = u_directionalLightColor.rgb * NdotL * u_directionalLightColor.a;

	// ALBEDO - DIFFUSE TEXTURE READ
	lowp vec4 albedo = texture2D(u_texture, v_texCoord0);
	albedo.rgb = albedo.rgb * albedo.rgb;
	
	// DETAIL MAP - TEXTURE READ
	lowp vec4 detail = texture2D(u_texture1, v_texCoord1);
	detail.rgb = detail.rgb * detail.rgb;
	
	//HSV SHIFTED DETAILS
	albedo.rgb = hsvShiftDetail(albedo.rgb, detail.r, u_HSV_shift.rgb);
	
	// LIGHTMAP - LIGHTMAP TEXTURE READ
	lowp vec4 lightMap = texture2D(u_texture2, v_texCoord2);
	lightMap.rgb = lightMap.rgb * lightMap.rgb;
	albedo.rgb = albedo.rgb * clamp(lightMap.rgb, v_ambient_colour, vec3(1.0,1.0,1.0));	
	albedo.rgb = mix(albedo.rgb, lightmap_color.rgb, (1.0-lightMap.r)*lightmap_color.a);
	
	// PHYSICALLY CORRECT SHADING
	lowp vec3 mrw = v_vert_color.rgb;

	// SPEC APPROXIMATION  
	lowp vec3 specColour = mix(vec3(0.03), albedo.rgb * 2.0, mrw.r);

	//ENV SAMPLING
	lowp vec4 envMapTex = texture2D(u_texture3, v_sphericalCoords);
	envMapTex = envMapTex * envMapTex;

	//Function CalculateEnvSpec accepts parameters (vec4 envMap, vec3 mrw, float NdotV, vec3 amb_color, vec3 spec_color)
	//Returns vec4 envSpec, env map colors in rgb, spec coefficient in a.
	lowp vec4 envSpec = CalculateEnvSpec(envMapTex, mrw, NdotV, v_ambient_colour, specColour);
	
	lowp vec3 spec_BRDF = CalculateBRDF(NdotV, mrw.g, specColour, dir_light, envSpec.a, albedo.a );


	// BUILD FRAGMENT OUTPUT  
	lowp vec4 returnValue = vec4(albedo.rgb * (dir_light + v_ambient_colour), 1.0);
	returnValue.rgb += spec_BRDF.rgb;
	returnValue.rgb += envSpec.rgb;

	// CHEAP FOG
	 float LdotV = (dot(L, V)+1.0) * 1.0;
	 lowp vec3 fogColour = u_fogColor.rgb;
	 returnValue.rgb = mix(returnValue.rgb, fogColour, v_vert_color.a);


	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = sqrt(returnValue.rgb);
	
	// ADD COLOR tO CLOSEDARKEN
	returnValue.rgb = mix((closeDarkenColor(u_closeDarken)* returnValue.rgb), returnValue.rgb, v_closeDarken);
	
	gl_FragColor = returnValue;
	}
